@@ -1,6 +1,7 @@
 ####################################################################################################
 
 import os
+import sys
 
 ####################################################################################################
 
@@ -8,6 +9,41 @@ import PySpice.Tools.Path as PathTools # Fixme: why ?
 
 ####################################################################################################
 
+class OsFactory:
+
+    ##############################################
+
+    def __init__(self):
+
+        if sys.platform.startswith('linux'):
+            self._name = 'linux'
+        elif sys.platform.startswith('win'):
+            self._name = 'windows'
+        elif sys.platform.startswith('darwin'):
+            self._name = 'osx'
+
+    ##############################################
+
+    @property
+    def name(self):
+        return self._name
+
+    @property
+    def on_linux(self):
+        return self._name == 'linux'
+
+    @property
+    def on_windows(self):
+        return self._name == 'windows'
+
+    @property
+    def on_osx(self):
+        return self._name == 'osx'
+
+OS = OsFactory()
+
+####################################################################################################
+
 _this_file = PathTools.to_absolute_path(__file__)
 
 class Path:
@@ -28,9 +64,3 @@ class Logging:
     def find(config_file):
 
         return PathTools.find(config_file, Logging.directories)
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
