@@ -19,46 +19,16 @@
 ####################################################################################################
 
 ####################################################################################################
-#
-# op: value
-# transient: time
-# dc: v-sweep
-# ac: frequency
-# sens: element parameters
-#
-# Probes
-#
-# Nodes:
-#   Voltage versus gnd
-#   real or complex
-#   static or (temporale/transient, v-sweep, frequency)
-#
-#   static waveform: (node->, unit, value)
-#   waveform: (node->, unit, values, abscissa->)
-#
-#   analysis.1.v !
-#   analysis.in.v !
-#   analysis['1'].v
-#   analysis.n1.v
-#
-# Source:
-#   Current
-#
-#   analysis.Vinput.i
-#
-# Vinput -> lower case
-#
-# axe.plot(x, y, *args, **kwargs)
-#
-###################################################################################################
 
-####################################################################################################
-#
-# Analysis versus Simulation
-#
+# Fixme: self._
+
+"""This module implements classes to handle analysis output.
+
+"""
+
 ####################################################################################################
 
-####################################################################################################
+import os
 
 import numpy as np
 
@@ -66,6 +36,21 @@ import numpy as np
 
 class WaveForm(np.ndarray):
 
+    """This class implements waveform on top of a Numpy Array.
+
+    Public Attributes:
+
+      :attr:`name`
+
+      :attr:`unit`
+
+      :attr:`title`
+
+      :attr:`abscissa`
+        Numpy array of the analysis abscissa
+
+    """
+
     ##############################################
 
     def __new__(cls, name, unit, data, title=None, abscissa=None):
@@ -73,7 +58,7 @@ class WaveForm(np.ndarray):
         obj = np.asarray(data).view(cls)
 
         obj.name = str(name)
-        obj.unit = str(unit)
+        obj.unit = unit
         obj.title = title # str(title)
         obj.abscissa = abscissa
 
@@ -95,7 +80,7 @@ class WaveForm(np.ndarray):
 
     def __repr__(self):
 
-        return 'variable {self.name} [{self.unit}]'.format(self=self)
+        return 'variable {0.name} [{0.unit}]'.format(self)
 
     ##############################################
 
@@ -104,7 +89,7 @@ class WaveForm(np.ndarray):
         if self.title is not None:
             return self.title
         else:
-            return '{self.name} [{self.unit}]'.format(self=self)
+            return '{0.name} [{0.unit}]'.format(self)
 
     ##############################################
 
@@ -116,12 +101,70 @@ class WaveForm(np.ndarray):
 
 class Analysis:
 
+    """Base class for the simulation output.
+
+    Depending of the simulation type, the simulator will return waveforms as a function of
+
+      * time
+      * frequency
+      * sweep
+      * ...
+
+    and corresponding to
+
+      * a node's voltage
+      * a source's current
+      * ...
+
+    The name of a waveform is
+
+      * node's voltage: node's name
+      * source's current: source'name
+      * ...
+
+    If the waveform name is a valid Python identifier, then you can get the corresponding waveform using::
+
+      analysis.waveforme_name
+
+    else you have to use this fallback::
+
+      analysis['waveforme_name']
+
+    Examples of usages::
+
+        # Operating point analysis
+        for node in analysis.nodes.values():
+            print('Node {}: {:5.2f} V'.format(str(node), float(node)))
+        for node in analysis.branches.values():
+            print('Node {}: {:5.2f} A'.format(str(node), float(node)))
+
+        # DC sensitivity analysis
+        for element in analysis.elements.values():
+            print(element, float(element))
+
+        # Plot the voltage of the "out" node
+        plt.plot(analysis.out.abscissa, analysis.out)
+
+    Public Attributes:
+
+      :attr:`nodes`
+        Dictionary for node voltages indexed by node names
+
+      :attr:`branches`
+        Dictionary for branch currents indexed by source names
+
+      :attr:`elements`
+        Dictionary for elements ...
+
+    """
+
     ##############################################
 
-    def __init__(self, nodes=(), branches=(), elements=()):
+    def __init__(self, simulation, nodes=(), branches=(), elements=()):
 
         # Fixme: branches are elements in fact, and elements is not yet supported ...
 
+        self._simulation = simulation
         # Fixme: to func?
         self.nodes = {waveform.name:waveform for waveform in nodes}
         self.branches = {waveform.name:waveform for waveform in branches}
@@ -129,7 +172,14 @@ class Analysis:
 
     ##############################################
 
-    def __getitem__(self, name):
+    @property
+    def simulation(self):
+        """Return the simulation instance"""
+        return self._simulation
+
+    ##############################################
+
+    def _get_item(self, name):
 
         if name in self.nodes:
             return self.nodes[name]
@@ -142,35 +192,57 @@ class Analysis:
 
     ##############################################
 
+    def __getitem__(self, name):
+
+        try:
+            return self._get_item(name)
+        except IndexError:
+            return self._get_item(name.lower())
+
+    ##############################################
+
+    @staticmethod
+    def _format_dict(d):
+
+        return os.linesep.join([' '*2 + str(x) for x in d])
+
+    ##############################################
+
     def __getattr__(self, name):
 
         try:
             return self.__getitem__(name)
         except IndexError:
-            raise AttributeError(name)
+            raise AttributeError(name + os.linesep +
+                                 'Nodes :' + os.linesep + self._format_dict(self.nodes) + os.linesep +
+                                 'Branches :' + os.linesep + self._format_dict(self.branches) + os.linesep +
+                                 'Elements :' + os.linesep + self._format_dict(self.elements))
 
 ####################################################################################################
 
 class OperatingPoint(Analysis):
+    """This class implements an operating point analysis."""
     pass
 
 ####################################################################################################
 
 class SensitivityAnalysis(Analysis):
 
+    """This class implements an sensitivity analysis."""
+
     ##############################################
 
-    def __init__(self, elements):
+    def __init__(self, simulation, elements):
 
-        super().__init__(elements=elements)
+        super().__init__(simulation=simulation, elements=elements)
 
 ####################################################################################################
 
 class DcAnalysis(Analysis):
 
-    """
+    """This class implements a DC analysis.
 
-    When the DC analysis is performed with multiple sources, v-sweep is the last source.
+    When the DC analysis is performed with multiple sources, sweep is the last source.
 
     The loop scheme is::
 
@@ -182,29 +254,30 @@ class DcAnalysis(Analysis):
 
     ##############################################
 
-    # Fixme: can be current sweep too
+    def __init__(self, simulation, sweep, nodes, branches):
 
-    def __init__(self, v_sweep, nodes, branches):
+        super().__init__(simulation=simulation, nodes=nodes, branches=branches)
 
-        super().__init__(nodes, branches)
-
-        self._v_sweep = v_sweep
+        self._sweep = sweep
 
     ##############################################
 
     @property
-    def v_sweep(self):
-        return self._v_sweep
+    def sweep(self):
+        """Return an Numpy array for the sweep abscissa"""
+        return self._sweep
 
 ####################################################################################################
 
 class AcAnalysis(Analysis):
 
+    """This class implements an AC analysis."""
+
     ##############################################
 
-    def __init__(self, frequency, nodes, branches):
+    def __init__(self, simulation, frequency, nodes, branches):
 
-        super().__init__(nodes, branches)
+        super().__init__(simulation=simulation, nodes=nodes, branches=branches)
 
         self._frequency = frequency
 
@@ -212,17 +285,20 @@ class AcAnalysis(Analysis):
 
     @property
     def frequency(self):
+        """Return an Numpy array for the frequency abscissa"""
         return self._frequency
 
 ####################################################################################################
 
 class TransientAnalysis(Analysis):
 
+    """This class implements a transient analysis."""
+
     ##############################################
 
-    def __init__(self, time, nodes, branches):
+    def __init__(self, simulation, time, nodes, branches):
 
-        super().__init__(nodes, branches)
+        super().__init__(simulation=simulation, nodes=nodes, branches=branches)
 
         self._time = time
 
@@ -230,10 +306,5 @@ class TransientAnalysis(Analysis):
 
     @property
     def time(self):
+        """Return an Numpy array for the time abscissa"""
         return self._time
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
