@@ -22,7 +22,8 @@
 
 ####################################################################################################
 
-from ..Unit.Units import Unit
+from ..Unit import Unit
+from ..Tools.StringTools import str_spice
 
 ####################################################################################################
 
@@ -144,7 +145,7 @@ class PositionalElementParameter(ParameterDescriptor):
     ##############################################
 
     def to_str(self, instance):
-        return str(self.__get__(instance))
+        return str_spice(self.__get__(instance))
 
     ##############################################
 
@@ -171,7 +172,6 @@ class ExpressionPositionalParameter(PositionalElementParameter):
     ##############################################
 
     def validate(self, value):
-
         return str(value)
 
 ####################################################################################################
@@ -182,6 +182,13 @@ class FloatPositionalParameter(PositionalElementParameter):
 
     ##############################################
 
+    def __init__(self, position, unit=None, **kwargs):
+
+        super().__init__(position, **kwargs)
+        self._unit = unit
+
+    ##############################################
+
     def validate(self, value):
 
         if isinstance(value, Unit):
@@ -279,7 +286,7 @@ class KeyValueParameter(ParameterDescriptor):
     ##############################################
 
     def str_value(self, instance):
-        return str(self.__get__(instance))
+        return str_spice(self.__get__(instance))
 
     ##############################################
 
@@ -319,7 +326,6 @@ class ExpressionKeyParameter(KeyValueParameter):
     ##############################################
 
     def validate(self, value):
-
         return str(value)
 
 ####################################################################################################
@@ -330,6 +336,13 @@ class FloatKeyParameter(KeyValueParameter):
 
     ##############################################
 
+    def __init__(self, spice_name, unit=None, **kwargs):
+
+        super().__init__(spice_name, **kwargs)
+        self._unit = unit
+
+    ##############################################
+
     def validate(self, value):
         return float(value)
 
@@ -378,9 +391,3 @@ class IntKeyParameter(KeyValueParameter):
 
     def validate(self, value):
         return int(value)
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
