@@ -20,15 +20,15 @@
 #
 ####################################################################################################
 
-""" This module implements high level elements built on Spice elements. """
+""" This module implements high level elements built on top of Spice elements. """
 
 # Fixme: these waveforms can be current sources as well
 
 ####################################################################################################
 
 from ..Math import rms_to_amplitude, amplitude_to_rms
-from ..Tools.StringTools import join_list, join_dict
-from ..Unit.Units import Frequency, Period
+from ..Tools.StringTools import join_list, join_dict, str_spice
+from ..Unit import as_s, as_V, as_A, as_Hz
 from .BasicElement import VoltageSource
 
 ####################################################################################################
@@ -89,12 +89,12 @@ class Sinusoidal(VoltageSource):
 
         super().__init__(name, node_plus, node_minus)
 
-        self.dc_offset = dc_offset
-        self.offset = offset
-        self.amplitude = amplitude
-        self.frequency = Frequency(frequency) # Fixme: protect by setter?
-        self.delay = delay
-        self.damping_factor = damping_factor
+        self.dc_offset = as_V(dc_offset)
+        self.offset = as_V(offset)
+        self.amplitude = as_V(amplitude)
+        self.frequency = as_Hz(frequency) # Fixme: protect by setter?
+        self.delay = as_s(delay)
+        self.damping_factor = as_Hz(damping_factor)
 
     ##############################################
 
@@ -112,10 +112,11 @@ class Sinusoidal(VoltageSource):
 
     def format_spice_parameters(self):
 
-        return join_list(('DC {}V'.format(self.dc_offset),
-                          'AC SIN({}V {}V {}Hz {}s {})'.format(self.offset, self.amplitude,
-                                                               self.frequency, self.delay,
-                                                               self.damping_factor)))
+        sin_part = join_list((self.offset, self.amplitude, self.frequency, self.delay, self.damping_factor))
+        return join_list((
+            'DC {}'.format(str_spice(self.dc_offset)),
+            'AC SIN({})'.format(sin_part),
+        ))
 
 ####################################################################################################
 
@@ -126,8 +127,8 @@ class AcLine(Sinusoidal):
     def __init__(self, name, node_plus, node_minus, rms_voltage=230, frequency=50):
 
         super().__init__(name, node_plus, node_minus,
-                                     amplitude=rms_to_amplitude(rms_voltage),
-                                     frequency=frequency)
+                         amplitude=rms_to_amplitude(rms_voltage),
+                         frequency=frequency)
 
 ####################################################################################################
 
@@ -211,13 +212,13 @@ class Pulse(VoltageSource):
 
         super().__init__(name, node_plus, node_minus)
 
-        self.initial_value = initial_value
-        self.pulsed_value = pulsed_value
-        self.delay_time = delay_time
-        self.rise_time = rise_time
-        self.fall_time = fall_time
-        self.pulse_width = pulse_width
-        self.period = Period(period) # Fixme: protect by setter?
+        self.initial_value = as_V(initial_value)
+        self.pulsed_value = as_V(pulsed_value)
+        self.delay_time = as_s(delay_time)
+        self.rise_time = as_s(rise_time)
+        self.fall_time = as_s(fall_time)
+        self.pulse_width = as_s(pulse_width)
+        self.period = as_s(period) # Fixme: protect by setter?
 
         # # Fixme: to func?
         # # Check parameters
@@ -302,12 +303,12 @@ class Exponential(VoltageSource):
 
         super().__init__(name, node_plus, node_minus)
 
-        self.initial_value = initial_value
-        self.pulsed_value = pulsed_value
-        self.rise_delay_time = rise_delay_time
-        self.rise_time_constant = rise_time_constant
-        self.fall_delay_time = fall_delay_time
-        self.fall_time_constant = fall_time_constant
+        self.initial_value = as_V(initial_value)
+        self.pulsed_value = as_V(pulsed_value)
+        self.rise_delay_time = as_s(rise_delay_time)
+        self.rise_time_constant = as_s(rise_time_constant)
+        self.fall_delay_time = as_s(fall_delay_time)
+        self.fall_time_constant = as_s(fall_time_constant)
 
     ##############################################
 
@@ -329,7 +330,7 @@ class PieceWiseLinear(VoltageSource):
 
     Spice Syntax::
 
-        PWL( T1 V1 <T2 V2 T3 V3 T4 V4 ... > ) <r=value> <td=value>    
+        PWL( T1 V1 <T2 V2 T3 V3 T4 V4 ... > ) <r=value> <td=value>
 
     Each pair of values (Ti , Vi) specifies that the value of the source is Vi (in Volts or Amps) at
     time = Ti . The value of the source at intermediate values of time is determined by using linear
@@ -354,9 +355,9 @@ class PieceWiseLinear(VoltageSource):
 
         super().__init__(name, node_plus, node_minus)
 
-        self.values = values
-        self.repeate_time = repeate_time
-        self.delay_time = delay_time
+        self.values = [as_V(x) for x in values]
+        self.repeate_time = as_s(repeate_time)
+        self.delay_time = as_s(delay_time)
 
     ##############################################
 
@@ -407,11 +408,11 @@ class SingleFrequencyFM(VoltageSource):
 
         super().__init__(name, node_plus, node_minus)
 
-        self.offset = offset
-        self.amplitude = amplitude
-        self.carrier_frequency = Frequency(carrier_frequency)
+        self.offset = as_V(offset)
+        self.amplitude = as_V(amplitude)
+        self.carrier_frequency = as_Hz(carrier_frequency)
         self.modulation_index = modulation_index
-        self.signal_frequency = Frequency(signal_frequency)
+        self.signal_frequency = as_Hz(signal_frequency)
 
     ##############################################
 
@@ -464,11 +465,11 @@ class AmplitudeModulated(VoltageSource):
 
         super().__init__(name, node_plus, node_minus)
 
-        self.offset = offset
-        self.amplitude = amplitude
-        self.carrier_frequency = Frequency(carrier_frequency)
-        self.modulating_frequency = Frequency(modulating_frequency)
-        self.signal_delay = signal_delay
+        self.offset = as_V(offset)
+        self.amplitude = as_V(amplitude)
+        self.carrier_frequency = as_Hz(carrier_frequency)
+        self.modulating_frequency = as_Hz(modulating_frequency)
+        self.signal_delay = as_s(signal_delay)
 
     ##############################################
 
@@ -526,8 +527,8 @@ class RandomVoltage(VoltageSource):
         super().__init__(name, node_plus, node_minus)
 
         self.random_type = random_type
-        self.duration = duration
-        self.time_delay = time_delay
+        self.duration = as_s(duration)
+        self.time_delay = as_s(time_delay)
         self.parameter1 = parameter1
         self.parameter2 = parameter2
 
@@ -551,9 +552,3 @@ class RandomVoltage(VoltageSource):
                 join_list((random_type, self.duration, self.time_delay,
                            self.parameter1, self.parameter2)) +
                 ')')
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
