@@ -22,6 +22,8 @@
 
 """This module implements a partial SPICE netlist parser.
 
+See the :command:`cir2py` tool for an example of usage of the parser.
+
 It would be difficult to implement a full parser for Ngspice since the syntax is mainly contextual.
 
 """
@@ -29,14 +31,13 @@ It would be difficult to implement a full parser for Ngspice since the syntax is
 ####################################################################################################
 
 import logging
+import os
 
 ####################################################################################################
 
-from .ElementParameter import (
-    FlagParameter,
-    )
-from .Netlist import ElementParameterMetaClass, NPinElement, Circuit
 from .BasicElement import SubCircuitElement, BipolarJunctionTransistor
+from .ElementParameter import FlagParameter
+from .Netlist import ElementParameterMetaClass, NPinElement, Circuit, SubCircuit
 
 ####################################################################################################
 
@@ -46,13 +47,15 @@ _module_logger = logging.getLogger(__name__)
 
 class PrefixData:
 
+    """This class represents a device prefix."""
+
     ##############################################
 
     def __init__(self, prefix, classes):
 
         self.prefix = prefix
         self.classes = classes
-        
+
         number_of_positionals_min = 1000
         number_of_positionals_max = 0
         has_optionals = False
@@ -61,11 +64,11 @@ class PrefixData:
             number_of_positionals_min = min(number_of_positionals_min, number_of_positionals)
             number_of_positionals_max = max(number_of_positionals_max, number_of_positionals)
             has_optionals = max(has_optionals, bool(element_class.optional_parameters))
-        
+
         self.number_of_positionals_min = number_of_positionals_min
         self.number_of_positionals_max = number_of_positionals_max
         self.has_optionals = has_optionals
-        
+
         self.multi_devices = len(classes) > 1
         self.npins = prefix in ('Q', 'X') # NPinElement, Q has 3 to 4 pins
         if self.npins:
@@ -73,7 +76,7 @@ class PrefixData:
         else:
             # Q and X are single
             self.number_of_pins = classes[0].number_of_pins
-        
+
         self.has_flag = False
         for element_class in classes:
             for parameter in element_class.optional_parameters.values():
@@ -101,8 +104,11 @@ class PrefixData:
 
 ####################################################################################################
 
-_prefix_cache = {prefix:PrefixData(prefix, classes)
-                 for prefix, classes in ElementParameterMetaClass.__classes__.items()}
+_prefix_cache = {}
+for prefix, classes in ElementParameterMetaClass.__classes__.items():
+    prefix_data = PrefixData(prefix, classes)
+    _prefix_cache[prefix] = prefix_data
+    _prefix_cache[prefix.lower()] = prefix_data
 
 # for prefix_data in sorted(_prefix_cache.values(), key=lambda x: len(x)):
 #     print(prefix_data.prefix,
@@ -139,30 +145,62 @@ _prefix_cache = {prefix:PrefixData(prefix, classes)
 
 ####################################################################################################
 
-class Token:
+class Statement:
 
-    """ This class implements a token, in fact a line in a Spice netlist. """
+    """ This class implements a statement, in fact a line in a Spice netlist. """
 
     ##############################################
 
-    def __init__(self, line):
+    def __init__(self, line, statement=None):
 
         self._line = line
+        self._line.lower_case_statement(statement)
 
     ##############################################
 
     def __repr__(self):
 
-        return "{} {}".format(self.__class__.__name__, repr(self._line))
+        return '{} {}'.format(self.__class__.__name__, repr(self._line))
+
+    ##############################################
+
+    def value_to_python(self, x):
+
+        if x:
+            if str(x)[0].isdigit():
+                return str(x)
+            else:
+                return "'{}'".format(x)
+        else:
+            return ''
+
+    ##############################################
+
+    def values_to_python(self, values):
+
+        return [self.value_to_python(x) for x in values]
+
+    ##############################################
+
+    def kwargs_to_python(self, kwargs):
+
+        return ['{}={}'.format(key, self.value_to_python(value))
+                for key, value in kwargs.items()]
+
+    ##############################################
+
+    def join_args(self, args):
+
+        return ', '.join(args)
 
 ####################################################################################################
 
-class Comment(Token):
+class Comment(Statement):
     pass
 
 ####################################################################################################
 
-class Title(Token):
+class Title(Statement):
 
     """ This class implements a title definition. """
 
@@ -170,7 +208,7 @@ class Title(Token):
 
     def __init__(self, line):
 
-        super().__init__(line)
+        super().__init__(line, statement='title')
         self._title = self._line.read_right_of('.title')
 
     ##############################################
@@ -181,11 +219,11 @@ class Title(Token):
     ##############################################
 
     def __repr__(self):
-        return "Title {}".format(self._title)
+        return 'Title {}'.format(self._title)
 
 ####################################################################################################
 
-class Include(Token):
+class Include(Statement):
 
     """ This class implements a include definition. """
 
@@ -193,7 +231,7 @@ class Include(Token):
 
     def __init__(self, line):
 
-        super().__init__(line)
+        super().__init__(line, statement='include')
         self._include = self._line.read_right_of('.include')
 
     ##############################################
@@ -204,11 +242,17 @@ class Include(Token):
     ##############################################
 
     def __repr__(self):
-        return "Include {}".format(self._title)
+        return 'Include {}'.format(self._include)
+
+    ##############################################
+
+    def to_python(self, netlist_name):
+
+        return '{}.include({})'.format(netlist_name, self._include) + os.linesep
 
 ####################################################################################################
 
-class Model(Token):
+class Model(Statement):
 
     """ This class implements a model definition.
 
@@ -222,7 +266,7 @@ class Model(Token):
 
     def __init__(self, line):
 
-        super().__init__(line)
+        super().__init__(line, statement='mode')
 
         # Fixme
         parameters, dict_parameters = self._line.split_line('.model')
@@ -240,11 +284,25 @@ class Model(Token):
 
     def __repr__(self):
 
-        return "Model {} {} {}".format(self._name, self._model_type, self._parameters)
+        return 'Model {} {} {}'.format(self._name, self._model_type, self._parameters)
+
+    ##############################################
+
+    def to_python(self, netlist_name):
+
+        args = self.values_to_python((self._name, self._model_type))
+        kwargs = self.kwargs_to_python(self._parameters)
+        return '{}.model({})'.format(netlist_name, self.join_args(args + kwargs)) + os.linesep
+
+    ##############################################
+
+    def build(self, circuit):
+
+        circuit.model(self._name, self._model_type, **self._parameters)
 
 ####################################################################################################
 
-class SubCircuit(Token):
+class SubCircuitStatement(Statement):
 
     """ This class implements a sub-circuit definition.
 
@@ -258,13 +316,13 @@ class SubCircuit(Token):
 
     def __init__(self, line):
 
-        super().__init__(line)
+        super().__init__(line, statement='subckt')
 
-        # Fixme:
+        # Fixme
         parameters, dict_parameters = self._line.split_line('.subckt')
         self._name, self._nodes = parameters[0], parameters[1:]
 
-        self._tokens = []
+        self._statements = []
 
     ##############################################
 
@@ -273,33 +331,57 @@ class SubCircuit(Token):
         """ Name of the sub-circuit. """
         return self._name
 
+    @property
+    def nodes(self):
+        """ Nodes of the sub-circuit. """
+        return self._nodes
+
     ##############################################
 
     def __repr__(self):
 
-        text = "SubCircuit {} {}\n".format(self._name, self._nodes)
-        text += '\n'.join(['  ' + repr(token) for token in self._tokens])
+        text = 'SubCircuit {} {}'.format(self._name, self._nodes) + os.linesep
+        text += os.linesep.join(['  ' + repr(statement) for statement in self._statements])
         return text
 
     ##############################################
 
     def __iter__(self):
 
-        """ Return an iterator on the tokens. """
+        """ Return an iterator on the statements. """
 
-        return iter(self._tokens)
+        return iter(self._statements)
 
     ##############################################
 
-    def append(self, token):
+    def append(self, statement):
 
-        """ Append a token to the token's list. """
+        """ Append a statement to the statement's list. """
 
-        self._tokens .append(token)
+        self._statements .append(statement)
+
+    ##############################################
+
+    def to_python(self, ground=0):
+
+        subcircuit_name = 'subcircuit_' + self._name
+        args = self.values_to_python([subcircuit_name] + self._nodes)
+        source_code = ''
+        source_code += '{} = SubCircuit({})'.format(subcircuit_name, self.join_args(args)) + os.linesep
+        source_code += SpiceParser.netlist_to_python(subcircuit_name, self, ground)
+        return source_code
+
+    ##############################################
+
+    def build(self, circuit):
+
+        subcircuit = SubCircuit(self._name, self._nodes)
+        SpiceParser.build_netlist(subcircuit, self._statements)
+        return subcircuit
 
 ####################################################################################################
 
-class Element(Token):
+class Element(Statement):
 
     """ This class implements an element definition.
 
@@ -314,10 +396,10 @@ class Element(Token):
     def __init__(self, line):
 
         super().__init__(line)
-        
+
         line_str = str(line)
-        # self._logger.debug('\n' + line_str)
-        
+        # self._logger.debug(os.linesep + line_str)
+
         # Retrieve device prefix
         self._prefix = line_str[0]
         prefix_data = _prefix_cache[self._prefix]
@@ -327,11 +409,11 @@ class Element(Token):
         stop_location = line_str.find(' ')
         # Fixme: if stop_location == -1:
         self._name = line_str[start_location:stop_location]
-        
+
         self._nodes = []
         self._parameters = []
         self._dict_parameters = {}
-        
+
         # Read nodes
         if not prefix_data.npins:
             number_of_pins = prefix_data.number_of_pins
@@ -345,7 +427,7 @@ class Element(Token):
                 args, stop_location = self._line.split_words(stop_location, until='=')
                 self._nodes = args[:-1]
                 self._parameters.append(args[-1]) # model name
-        
+
         # Read positionals
         number_of_positionals = prefix_data.number_of_positionals_min
         if number_of_positionals and stop_location is not None: # model is optional
@@ -353,11 +435,11 @@ class Element(Token):
         if prefix_data.multi_devices and stop_location is not None:
             remaining, stop_location = self._line.split_words(stop_location, until='=')
             self._parameters.extend(remaining)
-        
+
         if prefix_data.prefix in ('V', 'I') and stop_location is not None:
             # merge remaining
             self._parameters[-1] += line_str[stop_location:]
-        
+
         # Read optionals
         if prefix_data.has_optionals and stop_location is not None:
             kwargs, stop_location = self._line.split_words(stop_location)
@@ -370,8 +452,8 @@ class Element(Token):
                         self._dict_parameters['off'] = True
                     else:
                         self._logger.warn(line_str)
-                        # raise NameError("Bad element line:", line_str)
-        
+                        # raise NameError('Bad element line:', line_str)
+
         if prefix_data.multi_devices:
             for element_class in prefix_data:
                 if len(self._parameters) == element_class.number_of_positional_parameters:
@@ -379,7 +461,7 @@ class Element(Token):
         else:
             element_class = prefix_data.single
         self.factory = element_class
-        
+
         # Move positionals passed as kwarg
         to_delete = []
         for parameter in element_class.positional_parameters.values():
@@ -389,8 +471,8 @@ class Element(Token):
                 to_delete.append(i)
         for i in to_delete:
             del self._parameters[i]
-        
-        self._logger.debug('\n' + self.__repr__())
+
+        self._logger.debug(os.linesep + self.__repr__())
 
     ##############################################
 
@@ -403,7 +485,46 @@ class Element(Token):
 
     def __repr__(self):
 
-        return "Element {0._prefix} {0._name} {0._nodes} {0._parameters} {0._dict_parameters}".format(self)
+        return 'Element {0._prefix} {0._name} {0._nodes} {0._parameters} {0._dict_parameters}'.format(self)
+
+
+    ##############################################
+
+    def to_python(self, netlist_name, ground=0):
+
+        nodes = []
+        for node in self._nodes:
+            if str(node) == ground:
+                node = 0
+            nodes.append(node)
+        args = [self._name]
+        if self._prefix != 'X':
+            args += nodes + self._parameters
+        else: # != Spice
+            args += self._parameters + nodes
+        args = self.values_to_python(args)
+        kwargs = self.kwargs_to_python(self._dict_parameters)
+        return '{}.{}({})'.format(netlist_name, self._prefix, self.join_args(args + kwargs)) + os.linesep
+
+    ##############################################
+
+    def build(self, circuit, ground=0):
+
+        factory = getattr(circuit, self.factory.alias)
+        nodes = []
+        for node in self._nodes:
+            if str(node) == ground:
+                node = 0
+            nodes.append(node)
+        if self._prefix != 'X':
+            args = nodes + self._parameters
+        else: # != Spice
+            args = self._parameters + nodes
+        kwargs = self._dict_parameters
+        message = ' '.join([str(x) for x in (self._prefix, self._name, nodes,
+                                             self._parameters, self._dict_parameters)])
+        self._logger.debug(message)
+        factory(self._name, *args, **kwargs)
 
 ####################################################################################################
 
@@ -425,7 +546,7 @@ class Line:
             comment = text[location:]
         else:
             comment = ''
-        
+
         self._text = text
         self._comment = comment
         self._line_range = line_range
@@ -433,7 +554,7 @@ class Line:
     ##############################################
 
     def __repr__(self):
-        return "{0._line_range} {0._text}".format(self)
+        return '{0._line_range} {0._text}'.format(self)
 
     ##############################################
 
@@ -442,6 +563,13 @@ class Line:
 
     ##############################################
 
+    def lower_case_statement(self, statement):
+
+        if statement:
+            self._text = self._text.replace(statement.upper(), statement.lower())
+
+    ##############################################
+
     def read_right_of(self, text):
 
         return self._text[len(text):].strip()
@@ -463,16 +591,16 @@ class Line:
                 words.append(word)
             if stop_location is None: # we should stop
                 if number_of_words_read != number_of_words:
-                    template = "Bad element line, looking for word {}/{}:\n"
+                    template = 'Bad element line, looking for word {}/{}:' + os.linesep
                     raise NameError(template.format(number_of_words_read, number_of_words) +
-                                    line_str + '\n' +
+                                    line_str + os.linesep +
                                     ' '*start_location + '^')
             else:
                 if start_location < stop_location:
                     start_location = stop_location
                 else: # we have read a space
                     start_location += 1
-        
+
         return words, stop_location
 
     ##############################################
@@ -489,11 +617,11 @@ class Line:
                 if location != -1:
                     stop_location = location
                 else:
-                    raise NameError("Bad element line, missing key? " + line_str)
-        
+                    raise NameError('Bad element line, missing key? ' + line_str)
+
         line_str = line_str[start_location:stop_location]
         words = [x for x in line_str.split(' ') if x]
-        
+
         return words, stop_location
 
     ##############################################
@@ -507,15 +635,28 @@ class Line:
         Return the list of parameters and the dictionnary.
         """
 
-        raw_parameters = self._text[len(keyword):].split()
         parameters = []
         dict_parameters = {}
-        for parameter in raw_parameters:
-            if '=' in parameter:
-                key, value = parameter.split('=')
-                dict_parameters[key.strip()] = value.strip()
+
+        text = self._text[len(keyword):]
+        parts = []
+        for part in text.split():
+            if '=' in part:
+                left, right = [x for x in part.split('=')]
+                parts += [left, '=', right]
             else:
-                parameters.append(parameter)
+                parts.append(part)
+        i = 0
+        i_stop = len(parts)
+        while i < i_stop:
+            if i + 1 < i_stop and parts[i + 1] == '=':
+                key, value = parts[i], parts[i + 2]
+                dict_parameters[key] = value
+                i += 3
+            else:
+                parameters.append(parts[i])
+                i += 1
+
         return parameters, dict_parameters
 
 ####################################################################################################
@@ -546,13 +687,13 @@ class SpiceParser:
             with open(str(path), 'r') as f:
                 raw_lines = f.readlines()
         elif source is not None:
-            raw_lines = source.split('\n') # Fixme: other os
+            raw_lines = source.split(os.linesep) # Fixme: other os
         else:
             raise ValueError
-        
+
         lines = self._merge_lines(raw_lines)
         self._title = None
-        self._tokens = self._parse(lines)
+        self._statements = self._parse(lines)
         self._find_sections()
 
     ##############################################
@@ -560,7 +701,7 @@ class SpiceParser:
     def _merge_lines(self, raw_lines):
 
         """Merge broken lines and return a new list of lines.
-        
+
         A line starting with "+" continues the preceding line.
         """
 
@@ -584,13 +725,13 @@ class SpiceParser:
 
     def _parse(self, lines):
 
-        """ Parse the lines and return a list of tokens. """
+        """ Parse the lines and return a list of statements. """
 
-        tokens = []
+        statements = []
         sub_circuit = None
-        scope = tokens
+        scope = statements
         for line in lines:
-            # print repr(line)
+            # print(repr(line))
             text = str(line)
             lower_case_text = text.lower() # !
             if text.startswith('*'):
@@ -598,12 +739,12 @@ class SpiceParser:
             elif lower_case_text.startswith('.'):
                 lower_case_text = lower_case_text[1:]
                 if lower_case_text.startswith('subckt'):
-                    sub_circuit = SubCircuit(line)
-                    tokens.append(sub_circuit)
+                    sub_circuit = SubCircuitStatement(line)
+                    statements.append(sub_circuit)
                     scope = sub_circuit
                 elif lower_case_text.startswith('ends'):
                     sub_circuit = None
-                    scope = tokens
+                    scope = statements
                 elif lower_case_text.startswith('title'):
                     self._title = Title(line)
                     scope.append(self._title)
@@ -626,27 +767,27 @@ class SpiceParser:
                 element = Element(line)
                 scope.append(element)
 
-        return tokens
+        return statements
 
     ##############################################
 
     def _find_sections(self):
 
-        """ Look for model, sub-circuit and circuit definitions in the token list. """
+        """ Look for model, sub-circuit and circuit definitions in the statement list. """
 
         self.circuit = None
         self.subcircuits = []
         self.models = []
-        for token in self._tokens:
-            if isinstance(token, Title):
+        for statement in self._statements:
+            if isinstance(statement, Title):
                 if self.circuit is None:
-                    self.circuit = token
+                    self.circuit = statement
                 else:
-                    raise NameError("More than one title")
-            elif isinstance(token, SubCircuit):
-                self.subcircuits.append(token)
-            elif isinstance(token, Model):
-                self.models.append(token)
+                    raise NameError('More than one title')
+            elif isinstance(statement, SubCircuitStatement):
+                self.subcircuits.append(statement)
+            elif isinstance(statement, Model):
+                self.models.append(statement)
 
     ##############################################
 
@@ -662,87 +803,59 @@ class SpiceParser:
 
     ##############################################
 
+    @staticmethod
+    def build_netlist(netlist, statements):
+
+        for statement in statements:
+            if isinstance(statement, Include):
+                netlist.include(str(statement))
+
+        for statement in statements:
+            if isinstance(statement, Element):
+                statement.build(netlist, netlist.gnd)
+            elif isinstance(statement, Model):
+                statement.build(netlist)
+            elif isinstance(statement, SubCircuit):
+                subcircuit = statement.build()
+                netlist.subcircuit(subcircuit)
+
+    ##############################################
+
     def build_circuit(self, ground=0):
 
-        ground = str(ground)
-        
         circuit = Circuit(str(self._title))
-        
-        for token in self._tokens:
-            if isinstance(token, Include):
-                circuit.include(str(token))
-        
-        for token in self._tokens:
-            if isinstance(token, Element):
-                factory = getattr(circuit, token.factory.alias)
-                nodes = []
-                for node in token._nodes:
-                    if str(node) == ground:
-                        node = 0
-                    nodes.append(node)
-                if token._prefix != 'X':
-                    args = nodes + token._parameters
-                else: # != Spice
-                    args = token._parameters + nodes
-                kwargs = token._dict_parameters
-                message = ' '.join([str(x) for x in (token._prefix, token._name, nodes,
-                                                     token._parameters, token._dict_parameters)])
-                self._logger.debug(message)
-                factory(token._name, *args, **kwargs)
-        
+        self.build_netlist(circuit, self._statements)
         return circuit
 
     ##############################################
 
-    def _to_python(self, value):
+    @staticmethod
+    def netlist_to_python(netlist_name, statements, ground=0):
 
-        try:
-            int_value = int(value)
-            value = float(value)
-            if int_value == value:
-                return str(int_value)
-            else:
-                return str(value)
-        except ValueError:
-            return "'{}'".format(value)
+        source_code = ''
+        for statement in statements:
+            if isinstance(statement, Element):
+                source_code += statement.to_python(netlist_name, ground)
+            elif isinstance(statement, Include):
+                pass
+            elif isinstance(statement, Model):
+                source_code += statement.to_python(netlist_name)
+            elif isinstance(statement, SubCircuitStatement):
+                source_code += statement.to_python(netlist_name)
+            elif isinstance(statement, Include):
+                source_code += statement.to_python(netlist_name)
+        return source_code
 
     ##############################################
 
     def to_python_code(self, ground=0):
 
         ground = str(ground)
-        
-        # for token in self._tokens:
-        #     if isinstance(token, Include):
-        #         circuit.include(str(token))
 
-        if self._title:
-            title = self._title
-        else:
-            title = '...'
-        circuit = "circuit = Circuit('{}')\n".format(title)
-        
-        for token in self._tokens:
-            if isinstance(token, Element):
-                nodes = []
-                for node in token._nodes:
-                    if str(node) == ground:
-                        node = 0
-                    nodes.append(node)
-                if token._prefix != 'X':
-                    args = nodes + token._parameters
-                else: # != Spice
-                    args = token._parameters + nodes
-                args = [self._to_python(x) for x in args]
-                kwargs = ['{}={}'.format(key, self._to_python(value))
-                          for key, value in token._dict_parameters.items()]
-                parameters = ', '.join(args + kwargs)
-                circuit += "circuit.{}({})\n".format(token._prefix, parameters)
-        
-        return circuit
+        source_code = ''
 
-####################################################################################################
-#
-# End
-#
-####################################################################################################
+        if self.circuit:
+            source_code += "circuit = Circuit('{}')".format(self._title) + os.linesep
+        source_code += self.netlist_to_python('circuit', self._statements, ground)
+
+        return source_code
