@@ -18,29 +18,35 @@
 #
 ####################################################################################################
 
-"""
+####################################################################################################
+
+import os
+
+####################################################################################################
+
+"""This module provide tools to read the output of Ngspice.
 
 Header
 
-    .. code::
+.. code::
 
-        Circuit: 230V Rectifier
+    Circuit: 230V Rectifier
 
-        Doing analysis at TEMP = 25.000000 and TNOM = 25.000000
+    Doing analysis at TEMP = 25.000000 and TNOM = 25.000000
 
-        Title: 230V Rectifier
-        Date: Thu Jun  4 23:40:58  2015
-        Plotname: Transient Analysis
-        Flags: real
-        No. Variables: 6
-        No. Points: 0
-        Variables:
-        No. of Data Columns : 6
-                0       time    time
-                1       v(in)   voltage
-                ...
-                5       i(vinput)       current
-        Binary:
+    Title: 230V Rectifier
+    Date: Thu Jun  4 23:40:58  2015
+    Plotname: Transient Analysis
+    Flags: real
+    No. Variables: 6
+    No. Points: 0
+    Variables:
+    No. of Data Columns : 6
+            0       time    time
+            1       v(in)   voltage
+            ...
+            5       i(vinput)       current
+    Binary:
 
 Operating Point
 Node voltages and source branch currents:
@@ -81,6 +87,10 @@ Time, node voltages and source branch currents:
 
 ####################################################################################################
 
+from ..Unit import u_Degree, U_V, U_A, U_s, U_Hz
+
+####################################################################################################
+
 import logging
 import numpy as np
 
@@ -96,6 +106,8 @@ _module_logger = logging.getLogger(__name__)
 
 ####################################################################################################
 
+# Fixme: self._
+
 class Variable:
 
     """This class implements a variable or probe in a SPICE simulation output.
@@ -117,13 +129,13 @@ class Variable:
 
         self.index = int(index)
         self.name = str(name)
-        self.unit = str(unit) # could be guessed from name also for voltage node and branch current
+        self.unit = unit # could be guessed from name also for voltage node and branch current
         self.data = None
 
     ##############################################
 
     def __repr__(self):
-        return 'variable[{self.index}]: {self.name} [{self.unit}]'.format(self=self)
+        return 'variable[{0.index}]: {0.name} [{0.unit}]'.format(self)
 
     ##############################################
 
@@ -182,7 +194,7 @@ class Variable:
             data = data.real
         if to_float:
             data = float(data[0])
-        
+
         return WaveForm(self.simplified_name, self.unit, data, abscissa=abscissa)
 
 ####################################################################################################
@@ -227,29 +239,60 @@ class RawFile:
     def __init__(self, stdout, number_of_points):
 
         self.number_of_points = number_of_points
-        
+
         raw_data = self._read_header(stdout)
         self._read_variable_data(raw_data)
         # self._to_analysis()
 
+        self._simulation = None
+
+    ##############################################
+
+    @property
+    def simulation(self):
+
+        if self._simulation is not None:
+            return self._simulation
+        else:
+            raise NameError('Simulation is undefined')
+
+    @simulation.setter
+    def simulation(self, value):
+        self._simulation = value
+
+    ##############################################
+
+    @property
+    def circuit(self):
+        return self._simulation.circuit
+
+    ##############################################
+
+    __name_to_unit__ = {
+        'time': U_s,
+        'voltage': U_V,
+        'current': U_A,
+        'frequency': U_Hz,
+    }
+
     ##############################################
 
     def _read_header(self, stdout):
 
         """ Parse the header """
-        
-        binary_line = b'Binary:\n'
+
+        binary_line = b'Binary:' + os.linesep.encode('ascii')
         binary_location = stdout.find(binary_line)
         if binary_location < 0:
             raise NameError('Cannot locate binary data')
         raw_data_start = binary_location + len(binary_line)
-        # self._logger.debug('\n' + stdout[:raw_data_start].decode('utf-8'))
+        # self._logger.debug(os.linesep + stdout[:raw_data_start].decode('utf-8'))
         header_lines = stdout[:binary_location].splitlines()
         raw_data = stdout[raw_data_start:]
         header_line_iterator = iter(header_lines)
-        
-        self.circuit = self._read_header_field_line(header_line_iterator, 'Circuit')
-        self.temperature = self._read_header_line(header_line_iterator, 'Doing analysis at TEMP')
+
+        self.circuit_name = self._read_header_field_line(header_line_iterator, 'Circuit')
+        self.temperature, self.nominal_temperature = self._read_temperature_line(header_line_iterator)
         self.warnings = [self._read_header_field_line(header_line_iterator, 'Warning')
                          for i in range(stdout.count(b'Warning'))]
         for warning in self.warnings:
@@ -269,9 +312,11 @@ class RawFile:
             items = [x.strip() for x in line.split('\t') if x]
             # 0 frequency frequency grid=3
             index, name, unit = items[:3]
+            #  unit = time, voltage, current
+            unit = self.__name_to_unit__[unit] # convert to Unit
             self.variables[name] = Variable(index, name, unit)
         # self._read_header_field_line(header_line_iterator, 'Binary', has_value=False)
-        
+
         return raw_data
 
     ##############################################
@@ -295,7 +340,9 @@ class RawFile:
 
         line = self._read_line(header_line_iterator)
         self._logger.debug(line)
-        if not line.startswith(head_line):
+        if line.startswith(head_line):
+            return line
+        else:
             raise NameError("Unexpected line: %s" % (line))
 
     ##############################################
@@ -322,6 +369,27 @@ class RawFile:
 
     ##############################################
 
+    def _read_temperature_line(self, header_line_iterator):
+
+        # Doing analysis at TEMP = 25.000000 and TNOM = 25.000000
+
+        line = self._read_header_line(header_line_iterator, 'Doing analysis at TEMP')
+        pattern1 = 'TEMP = '
+        pattern2 = ' and TNOM = '
+        pos1 = line.find(pattern1)
+        pos2 = line.find(pattern2)
+        if pos1 != -1 and pos2 != -1:
+            part1 = line[pos1+len(pattern1):pos2]
+            part2 = line[pos2+len(pattern2):].strip()
+            temperature = u_Degree(float(part1))
+            nominal_temperature = u_Degree(float(part2))
+        else:
+            temperature = None
+            nominal_temperature = None
+        return temperature, nominal_temperature
+
+    ##############################################
+
     def _read_variable_data(self, raw_data):
 
         """ Read the raw data and set the variable values. """
@@ -332,7 +400,7 @@ class RawFile:
             number_of_columns = 2*self.number_of_variables
         else:
             raise NotImplementedError
-        
+
         input_data = np.fromstring(raw_data, count=number_of_columns*self.number_of_points, dtype='f8')
         input_data = input_data.reshape((self.number_of_points, number_of_columns))
         input_data = input_data.transpose()
@@ -346,10 +414,11 @@ class RawFile:
 
     ##############################################
 
-    def fix_case(self, circuit):
+    def fix_case(self):
 
         """ Ngspice return lower case names. This method fixes the case of the variable names. """
 
+        circuit = self.circuit
         element_translation = {element.lower():element for element in circuit.element_names()}
         node_translation = {node.lower():node for node in circuit.node_names()}
         for variable in self.variables.values():
@@ -380,10 +449,10 @@ class RawFile:
 
     ##############################################
 
-    def to_analysis(self, circuit):
+    def to_analysis(self):
+
+        self.fix_case()
 
-        self.fix_case(circuit)
-        
         if self.plot_name == 'Operating Point':
             return self._to_operating_point_analysis()
         elif self.plot_name == 'Sensitivity Analysis':
@@ -395,13 +464,18 @@ class RawFile:
         elif self.plot_name == 'Transient Analysis':
             return self._to_transient_analysis()
         else:
+
             raise NotImplementedError("Unsupported plot name {}".format(self.plot_name))
 
     ##############################################
 
     def _to_operating_point_analysis(self):
 
-        return OperatingPoint(nodes=self.nodes(to_float=True), branches=self.branches(to_float=True))
+        return OperatingPoint(
+            simulation=self.simulation,
+            nodes=self.nodes(to_float=True),
+            branches=self.branches(to_float=True),
+        )
 
     ##############################################
 
@@ -409,7 +483,10 @@ class RawFile:
 
         # Fixme: test .SENS I (VTEST)
         # Fixme: separate v(vinput), analysis.R2.m
-        return SensitivityAnalysis(elements=self.elements())
+        return SensitivityAnalysis(
+            simulation=self.simulation,
+            elements=self.elements(),
+        )
 
     ##############################################
 
@@ -420,27 +497,36 @@ class RawFile:
         elif 'v(i-sweep)' in self.variables:
             sweep_variable = self.variables['v(i-sweep)']
         else:
-            # 
+            #
             raise NotImplementedError
         sweep = sweep_variable.to_waveform()
-        return DcAnalysis(sweep, nodes=self.nodes(), branches=self.branches())
+        return DcAnalysis(
+            simulation=self.simulation,
+            sweep=sweep,
+            nodes=self.nodes(),
+            branches=self.branches(),
+        )
 
     ##############################################
 
     def _to_ac_analysis(self):
 
         frequency = self.variables['frequency'].to_waveform(to_real=True)
-        return AcAnalysis(frequency, nodes=self.nodes(), branches=self.branches())
+        return AcAnalysis(
+            simulation=self.simulation,
+            frequency=frequency,
+            nodes=self.nodes(),
+            branches=self.branches(),
+        )
 
     ##############################################
 
     def _to_transient_analysis(self):
 
         time = self.variables['time'].to_waveform(to_real=True)
-        return TransientAnalysis(time, nodes=self.nodes(abscissa=time), branches=self.branches(abscissa=time))
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
+        return TransientAnalysis(
+            simulation=self.simulation,
+            time=time,
+            nodes=self.nodes(abscissa=time),
+            branches=self.branches(abscissa=time),
+        )
