@@ -1,5 +1,5 @@
-#!# This example depicts a voltage multiplier using diodes and capacitors.
-#!# See `link <http://en.wikipedia.org/wiki/Voltage_multiplier>`_
+#!# This example depicts a voltage multiplier using diodes and capacitors.  To go further, you can
+#!# read this `page <http://en.wikipedia.org/wiki/Voltage_multiplier>`_ on Wikipedia.
 
 ####################################################################################################
 
@@ -17,7 +17,7 @@ logger = Logging.setup_logging()
 from PySpice.Probe.Plot import plot
 from PySpice.Spice.Library import SpiceLibrary
 from PySpice.Spice.Netlist import Circuit
-from PySpice.Unit.Units import *
+from PySpice.Unit import *
 
 ####################################################################################################
 
@@ -30,7 +30,7 @@ spice_library = SpiceLibrary(libraries_path)
 
 circuit = Circuit('Voltage Multiplier')
 circuit.include(spice_library['1N4148'])
-source = circuit.Sinusoidal('input', 'in', circuit.gnd, amplitude=10, frequency=50)
+source = circuit.Sinusoidal('input', 'in', circuit.gnd, amplitude=10@u_V, frequency=50@u_Hz)
 
 multiplier = 5
 for i in range(multiplier):
@@ -39,9 +39,9 @@ for i in range(multiplier):
     else:
         top_node = 'in'
     midlle_node, bottom_node = i + 1, i
-    circuit.C(i, top_node, midlle_node, milli(1))
+    circuit.C(i, top_node, midlle_node, 1@u_mF)
     circuit.X(i, '1N4148', midlle_node, bottom_node)
-circuit.R(1, multiplier, multiplier+1, mega(1))
+circuit.R(1, multiplier, multiplier+1, 1@u_MΩ)
 
 simulator = circuit.simulator(temperature=25, nominal_temperature=25)
 analysis = simulator.transient(step_time=source.period/200, end_time=source.period*20)
@@ -63,7 +63,7 @@ for i in range(1, multiplier+1):
         y -= analysis['in']
     plot(y, axis=axe)
 # axe.axhline(-multiplier*source.amplitude)
-axe.set_ylim(-multiplier*1.1*source.amplitude, 1.1*source.amplitude)
+axe.set_ylim(float(-multiplier*1.1*source.amplitude), float(1.1*source.amplitude))
 axe.legend(['input'] + ['*' + str(i) for i in range(1, multiplier+1)] ,
            loc=(.2,.8))
 
@@ -71,9 +71,3 @@ plt.tight_layout()
 plt.show()
 
 #fig# save_figure(figure, 'voltage-multiplier.png')
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
