@@ -1,3 +1,5 @@
+#!# This example shows how to simulate and plot the characteristic curve of a Zener diode.
+
 ####################################################################################################
 
 import os
@@ -14,7 +16,7 @@ logger = Logging.setup_logging()
 
 from PySpice.Spice.Netlist import Circuit
 from PySpice.Spice.Library import SpiceLibrary
-from PySpice.Unit.Units import *
+from PySpice.Unit import *
 
 ####################################################################################################
 
@@ -23,14 +25,16 @@ spice_library = SpiceLibrary(libraries_path)
 
 ####################################################################################################
 
+#cm# zener-diode-characteristic-curve-circuit.m4
+
 circuit = Circuit('Diode DC Curve')
 
 circuit.include(spice_library['1N4148'])
 # 1N5919B: 5.6 V, 3.0 W Zener Diode Voltage Regulator
 circuit.include(spice_library['d1n5919brl'])
 
-circuit.V('input', 'in', circuit.gnd, '10V')
-circuit.R(1, 'in', 'out', 1) # not required for simulation
+circuit.V('input', 'in', circuit.gnd, 10@u_V)
+circuit.R(1, 'in', 'out', 1@u_Ω) # not required for simulation
 # circuit.X('D1', '1N4148', 'out', circuit.gnd)
 circuit.X('DZ1', 'd1n5919brl', 'out', circuit.gnd)
 
@@ -76,9 +80,3 @@ plt.tight_layout()
 plt.show()
 
 #fig# save_figure(figure, 'zener-characteristic-curve.png')
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
