@@ -1,5 +1,8 @@
+#!# This example shows a low-pass RC Filter.
+
 ####################################################################################################
 
+import math
 import numpy as np
 import matplotlib.pyplot as plt
 
@@ -12,24 +15,34 @@ logger = Logging.setup_logging()
 
 from PySpice.Plot.BodeDiagram import bode_diagram
 from PySpice.Spice.Netlist import Circuit
-from PySpice.Unit.Units import *
+from PySpice.Unit import *
 
 ####################################################################################################
 
+#cm# low-pass-rc-filter.m4
+
 circuit = Circuit('Low-Pass RC Filter')
 
-circuit.Sinusoidal('input', 'in', circuit.gnd, amplitude=1)
-circuit.R('f', 'in', 'out', kilo(1))
-circuit.C('f', 'out', circuit.gnd, micro(1))
+circuit.Sinusoidal('input', 'in', circuit.gnd, amplitude=1@u_V)
+R1 = circuit.R(1, 'in', 'out', 1@u_kΩ)
+C1 = circuit.C(1, 'out', circuit.gnd, 1@u_uF)
+
+#!# The break frequency is given by :math:`f_c = \frac{1}{2 \pi R C}`
+
+break_frequency = 1 / (2 * math.pi * float(R1.resistance * C1.capacitance))
+print("Break frequency = {:.1f} Hz".format(break_frequency))
+#o#
 
 simulator = circuit.simulator(temperature=25, nominal_temperature=25)
-analysis = simulator.ac(start_frequency=1, stop_frequency=mega(1), number_of_points=10,  variation='dec')
+analysis = simulator.ac(start_frequency=1@u_Hz, stop_frequency=1@u_MHz, number_of_points=10,  variation='dec')
+# print(analysis.out)
 
-print(analysis.out)
+#!# We plot the Bode diagram.
 
 figure = plt.figure(1, (20, 10))
 plt.title("Bode Diagram of a Low-Pass RC Filter")
-bode_diagram(axes=(plt.subplot(211), plt.subplot(212)),
+axes = (plt.subplot(211), plt.subplot(212))
+bode_diagram(axes=axes,
              frequency=analysis.frequency,
              gain=20*np.log10(np.absolute(analysis.out)),
              phase=np.angle(analysis.out, deg=False),
@@ -37,12 +50,10 @@ bode_diagram(axes=(plt.subplot(211), plt.subplot(212)),
              color='blue',
              linestyle='-',
          )
+for axe in axes:
+    axe.axvline(x=break_frequency, color='red')
+
 plt.tight_layout()
 plt.show()
-#fig# save_figure(figure, 'low-pass-rc-filter.png')
 
-####################################################################################################
-#
-# End
-#
-####################################################################################################
+#fig# save_figure(figure, 'low-pass-rc-filter-bode-diagram.png')
