@@ -1,3 +1,5 @@
+#!# This example shows the computation of the DC biases in a resistor bridge.
+
 ####################################################################################################
 
 import PySpice.Logging.Logging as Logging
@@ -6,18 +8,20 @@ logger = Logging.setup_logging()
 ####################################################################################################
 
 from PySpice.Spice.Netlist import Circuit
-from PySpice.Unit.Units import *
+from PySpice.Unit import *
 
 ####################################################################################################
 
+#cm# resistor-bridge.m4
+
 circuit = Circuit('Resistor Bridge')
 
-circuit.V('input', 1, circuit.gnd, 10) # Fixme: V(10) uV(10) 10*V
-circuit.R(1, 1, 2, kilo(2))
-circuit.R(2, 1, 3, kilo(1))
-circuit.R(3, 2, circuit.gnd, kilo(1))
-circuit.R(4, 3, circuit.gnd, kilo(2))
-circuit.R(5, 3, 2, kilo(2))
+circuit.V('input', 1, circuit.gnd, 10@u_V)
+circuit.R(1, 1, 2, 2@u_kΩ)
+circuit.R(2, 1, 3, 1@u_kΩ)
+circuit.R(3, 2, circuit.gnd, 1@u_kΩ)
+circuit.R(4, 3, circuit.gnd, 2@u_kΩ)
+circuit.R(5, 3, 2, 2@u_kΩ)
 
 simulator = circuit.simulator(temperature=25, nominal_temperature=25)
 analysis = simulator.operating_point()
@@ -25,9 +29,3 @@ analysis = simulator.operating_point()
 for node in analysis.nodes.values():
     print('Node {}: {:4.1f} V'.format(str(node), float(node))) # Fixme: format value + unit
 #o#
-
-####################################################################################################
-#
-# End
-#
-####################################################################################################
