@@ -1,3 +1,5 @@
+#!# This example shows the computation of the DC bias and sensitivity in a voltage divider.
+
 ####################################################################################################
 
 import PySpice.Logging.Logging as Logging
@@ -6,15 +8,17 @@ logger = Logging.setup_logging()
 ####################################################################################################
 
 from PySpice.Spice.Netlist import Circuit
-from PySpice.Unit.Units import *
+from PySpice.Unit import *
 
 ####################################################################################################
 
+#cm# voltage-divider.m4
+
 circuit = Circuit('Voltage Divider')
 
-circuit.V('input', 'in', circuit.gnd, '10V')
-circuit.R(1, 'in', 'out', kilo(9))
-circuit.R(2, 'out', circuit.gnd, kilo(1))
+circuit.V('input', 'in', circuit.gnd, 10@u_V)
+circuit.R(1, 'in', 'out', 9@u_kΩ)
+circuit.R(2, 'out', circuit.gnd, 1@u_kΩ)
 
 ####################################################################################################
 
@@ -23,13 +27,9 @@ simulator = circuit.simulator(temperature=25, nominal_temperature=25)
 analysis = simulator.operating_point()
 for node in (analysis['in'], analysis.out): # .in is invalid !
     print('Node {}: {} V'.format(str(node), float(node)))
+#o#
 
 analysis = simulator.dc_sensitivity('v(out)')
 for element in analysis.elements.values():
     print(element, float(element))
-
-####################################################################################################
-# 
-# End
-# 
-####################################################################################################
+#o#
