@@ -24,9 +24,6 @@ import os
 import sys
 import subprocess
 
-from distutils.sysconfig import get_python_lib
-site_packages_path = get_python_lib()
-
 ####################################################################################################
 
 def merge_include(src_lines, doc_path, included_rst_files=None):
@@ -49,7 +46,7 @@ def merge_include(src_lines, doc_path, included_rst_files=None):
 
 # Utility function to read the README file.
 # Used for the long_description.
-def read(file_name):
+def read_readme(file_name):
 
     source_path = os.path.dirname(os.path.realpath(__file__))
     if os.path.basename(source_path) == 'tools':
@@ -68,7 +65,7 @@ def read(file_name):
 
 ####################################################################################################
 
-long_description = read('README.txt')
+long_description = read_readme('README.txt')
 
 ####################################################################################################
 
@@ -78,17 +75,14 @@ try:
 except FileNotFoundError:
     sys.stderr.write('\n\nWarning: You must install ngspice\n\n')
 
-PySpice_path = os.path.join(site_packages_path, 'PySpice')
-
 setup_dict = dict(
     name='PySpice',
-    version='0.3.2',
+    version='1.0.0',
     author='Fabrice Salvaire',
     author_email='fabrice.salvaire@orange.fr',
-    description='PySpice is a Python 3 Package to generate and steer Berkeley Spice circuit, '
-    ' to simulate them and finally analyse the output using Python.',
-    license="GPLv3",
-    keywords="spice, berkeley, ngspice, circuit, simulation, electronic",
+    description='Simulate electronic circuit using Python and Ngspice simulator',
+    license='GPLv3',
+    keywords= 'spice berkeley ngspice electronic circuit simulation simulator',
     url='https://github.com/FabriceSalvaire/PySpice',
     scripts=['bin/cir2py'],
     packages=['PySpice', # Fixme:
@@ -104,24 +98,22 @@ setup_dict = dict(
               'PySpice.Unit',
           ],
     # package_dir = {'PySpice': 'PySpice'},
-    data_files=[
-        (os.path.join(PySpice_path, 'Config'),
-         [os.path.join('PySpice', 'Config', 'logging.yml')]),
-        (os.path.join(PySpice_path, 'Spice', 'NgSpice'),
-         [os.path.join('PySpice', 'Spice', 'NgSpice', 'api.h')]),
-    ],
+    package_data={
+        'PySpice.Config': ['logging.yml'],
+        'PySpice.Spice.NgSpice': ['api.h'],
+    },
     long_description=long_description,
     # cf. http://pypi.python.org/pypi?%3Aaction=list_classifiers
     classifiers=[
-        "Topic :: Scientific/Engineering",
-        "Intended Audience :: Education",
-        "Development Status :: 5 - Production/Stable",
-        "License :: OSI Approved :: GNU General Public License (GPL)",
-        "Operating System :: OS Independent",
-        "Programming Language :: Python :: 3.4",
+        'Topic :: Scientific/Engineering',
+        'Intended Audience :: Education',
+        'Development Status :: 5 - Production/Stable',
+        'License :: OSI Approved :: GNU General Public License (GPL)',
+        'Operating System :: OS Independent',
+        'Programming Language :: Python :: 3.5',
         ],
     requires=[
-        'PyYAML'
+        'PyYAML',
         'cffi',
         'matplotlib',
         'numpy',
